<?php

declare(strict_types=1);

namespace Drupal\paragraph_lineage\Controller;

use Drupal\Component\Render\MarkupInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityDisplayRepositoryInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Link;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\paragraphs\ParagraphInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Returns responses for Paragraph Lineage routes.
 */
final class ParagraphLineageViewController extends ControllerBase {

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected RouteMatchInterface $route_match;

  /**
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  private EntityTypeManagerInterface $entity_type_manager;

  /**
   * @var \Drupal\Core\Entity\EntityDisplayRepositoryInterface
   */
  private EntityDisplayRepositoryInterface $entity_display_repository;

  /**
   * Constructs a ParagraphViewController.
   *
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The renderer service.
   */
  public function __construct(RendererInterface $renderer, RouteMatchInterface $route_match, EntityTypeManagerInterface $entity_type_manager, EntityDisplayRepositoryInterface $entity_display_repository) {
    $this->renderer = $renderer;
    $this->route_match = $route_match;
    $this->entity_type_manager = $entity_type_manager;
    $this->entity_display_repository = $entity_display_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): ParagraphLineageViewController {
    /** @var \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager */
    $entity_type_manager = $container->get('entity_type.manager');
    /** @var RendererInterface $renderer */
    $renderer = $container->get('renderer');
    $route_match = \Drupal::routeMatch();
    /** @var EntityDisplayRepositoryInterface $entity_display_repository */
    $entity_display_repository = $container->get('entity_display.repository');
    return new ParagraphLineageViewController(
      $renderer,
      $route_match,
      $entity_type_manager,
      $entity_display_repository,
    );
  }

  /**
   * Builds the response.
   *
   * @throws \Drupal\Core\Entity\EntityMalformedException
   */
  public function build(): array {
    // Get the paragraph from the route.
    /** @var \Drupal\paragraphs\ParagraphInterface $paragraph */
    $paragraph = $this->route_match->getParameter('paragraph');
    if (!$paragraph instanceof ParagraphInterface) {
      throw new \InvalidArgumentException('No paragraph found in the route.');
    }

    // Determine if a teaser or preview mode is identified for the $paragraph->bundle().
    $rendered_paragraph = $this->renderEntityPreview($paragraph);

    $lineage = [];
    // Get the parent of the entity;
    $parent = $paragraph->getParentEntity();
    if ($parent instanceof EntityInterface)  {
      $this->getLineage($lineage, $parent);
    }
    return [
      '#theme' => 'paragraph_lineage',
      '#paragraph' => [
        'type' => $paragraph->getEntityTypeId(),
        'bundle' => $paragraph->bundle(),
        'label' => $paragraph->label(),
        'content' => $rendered_paragraph,
      ],
      '#lineage' => $lineage,
    ];
  }

  /**
   * Recursively retrieves the lineage of an entity.
   *
   * @param array &$lineage
   *    A renderable array of the lineage of the entity.
   *
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity for which to retrieve the lineage.
   *
   * @return void
   *
   * This function generates a renderable array of links representing the
   * hierarchy of parent entities for a given entity. If the entity is a
   * paragraph, it traverses up the hierarchy to include all parent entities.
   * For non-paragraph entities, it simply retrieves the canonical route link.
   */
  protected function getLineage(array &$lineage, EntityInterface $entity): void {
    $route = 'entity.' . $entity->getEntityTypeId() . '.canonical';
    $parent =  NULL;
    if ($entity instanceof ParagraphInterface) {
        $route = 'entity.paragraph.canonical';
        $parent = $entity->getParentEntity();
    }

    $lineage[] = [
      'type' => $entity->getEntityTypeId(),
      'bundle' => $entity->bundle(),
      'content' => $this->renderEntityPreview($entity),
      'link' => Link::createFromRoute($entity->label(), $route, [$entity->getEntityTypeId() => $entity->id()])
        ->toRenderable(),
    ];
    if($parent instanceof EntityInterface) {
      $this->getLineage($lineage, $parent);
    }

  }

  /**
   * Renders an entity in a preview mode if available..
   *
   * @param \Drupal\paragraphs\ParagraphInterface $entity
   *
   * @return \Drupal\Component\Render\MarkupInterface
   *
   * This function renders an entity in a preview mode if available. If a
   *   preview mode is not available, it falls back to the default view mode.
   */
  protected function renderEntityPreview(EntityInterface $entity): MarkupInterface {
    $view_mode = 'default';
    $view_modes = $this->entity_display_repository->getViewModes($entity->bundle());
    // Check if preview or teaser are available. If so, set that as the view mode.
    if (isset($view_modes['preview'])) {
      $view_mode = 'preview';
    }
    else {
      if (isset($view_modes['teaser'])) {
        $view_mode = 'teaser';
      }
    }

    // Render the entity using the defined view mode.
    $view = $this->entity_type_manager
      ->getViewBuilder($entity->getEntityTypeId())
      ->view($entity, $view_mode);
    return $this->renderer->render($view);
  }

}
